subroutine Interaction(a2,b2,Rcut2,Ucut,Ca,Cb,Db,Da,Position,M,Correction,L,CG,DG,Cntr,Sprd,U,W,d,N0,M0)
        
    implicit none
    
    integer(4), intent(in) :: d        
    integer(4), intent(in) :: N0
    integer(4), intent(in) :: M0
    integer(4), intent(in) :: a2,b2
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Rcut2
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Ucut
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Ca,Cb,Db,Da
    real(8), intent(in), dimension(0:d-1,0:N0-1) :: Position
    integer(4), intent(in), dimension(0:N0-1) :: M
    real(8), intent(in) :: Correction          
    real(8), intent(in) :: L
    real(8), intent(in) :: CG,DG
    real(8), intent(in) :: Cntr,Sprd
    real(8), intent(out) :: U,W

    integer(4) :: i,j  ! dummy indecies
    real(8), dimension(d) :: Ri  ! position of a single molecule
    real(8), dimension(d) :: Rij  ! relative position between a pair of molecules
    real(8) :: Rij2  ! relative position between a pair of molecules (squared value)
    real(8) :: dRij  ! relative position between a pair of molecules (modified value)
    real(8) :: LJ_Aij,LJ_Bij  ! value of Lennard-Jones interaction for a pair of molecules
    real(8) :: Gauss0ij  ! value of Gaussian interaction for a pair of molecules   
            
                 
    U=0
    W=0
            
    do i=0, N0-2  ! go through all molecules
        Ri(:)=Position(:,i)
        do j=i+1, N0-1  ! go through all other molecules
            Rij(:)=Ri(:)-Position(:,j)
            Rij(:)=Rij(:)-L*nint(Rij(:)/L)
            Rij2=dot_product(Rij,Rij)
            if (Rij2<Rcut2(M(i),M(j))) then  ! calculate interaction  
                LJ_Aij=1./(Rij2**a2)
                LJ_Bij=1./(Rij2**b2)
                U=U+Ca(M(i),M(j))*LJ_Aij-Cb(M(i),M(j))*LJ_Bij-Ucut(M(i),M(j))
                W=W+Da(M(i),M(j))*LJ_Aij-Db(M(i),M(j))*LJ_Bij
                if (M(i)*M(j)==1) then
                    dRij=sqrt(Rij2)-Cntr
                    Gauss0ij=exp(-Sprd*(dRij*dRij))
                    U=U+CG*Gauss0ij
                    W=W+DG*dRij*(dRij+Cntr)*Gauss0ij
                end if
            end if   
        end do
    end do
            
    U=U+Correction
    W=W+Correction
            
end subroutine


subroutine Interaction0(a2,b2,i,Rcut2,Ucut,Ca,Cb,Db,Da,Position,Ri,M,Correction,L,CG,DG,Cntr,Sprd,U,W,d,N0,M0)
        
    implicit none
    
    integer(4), intent(in) :: d        
    integer(4), intent(in) :: N0
    integer(4), intent(in) :: M0
    integer(4), intent(in) :: a2,b2
    integer(4), intent(in) :: i
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Rcut2
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Ucut
    real(8), intent(in), dimension(0:M0-1,0:M0-1) :: Ca,Cb,Db,Da
    real(8), intent(in), dimension(0:d-1,0:N0-1) :: Position
    real(8), intent(in), dimension(0:d-1) :: Ri
    integer(4), intent(in), dimension(0:N0-1) :: M
    real(8), intent(in) :: Correction          
    real(8), intent(in) :: L
    real(8), intent(in) :: CG,DG
    real(8), intent(in) :: Cntr,Sprd
    real(8), intent(out) :: U,W

    integer(4) :: j  ! dummy indecies
    real(8), dimension(d) :: Rij  ! relative position between a pair of molecules
    real(8) :: Rij2  ! relative position between a pair of molecules (squared value)
    real(8) :: dRij  ! relative position between a pair of molecules (modified value)
    real(8) :: LJ_Aij,LJ_Bij  ! value of Lennard-Jones interaction for a pair of molecules
    real(8) :: Gauss0ij  ! value of Gaussian interaction for a pair of molecules   
            
                 
    U=0
    W=0
            
    do j=0, N0-1  ! go through all molecules
        if (j/=i) then
            Rij(:)=Ri(:)-Position(:,j)
            Rij(:)=Rij(:)-L*nint(Rij(:)/L)
            Rij2=dot_product(Rij,Rij)
            if (Rij2<Rcut2(M(i),M(j))) then  ! calculate interaction  
                LJ_Aij=1./(Rij2**a2)
                LJ_Bij=1./(Rij2**b2)
                U=U+Ca(M(i),M(j))*LJ_Aij-Cb(M(i),M(j))*LJ_Bij-Ucut(M(i),M(j))
                W=W+Da(M(i),M(j))*LJ_Aij-Db(M(i),M(j))*LJ_Bij
                if (M(i)*M(j)==1) then
                    dRij=sqrt(Rij2)-Cntr
                    Gauss0ij=exp(-Sprd*(dRij*dRij))
                    U=U+CG*Gauss0ij
                    W=W+DG*dRij*(dRij+Cntr)*Gauss0ij
                end if
            end if   
        end if
    end do
            
end subroutine